/********************************************************************
 FileName:		user.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18 or PIC24 USB Microcontrollers
 Hardware:		The code is natively intended to be used on the following
 				hardware platforms: PICDEM FS USB Demo Board, 
 				PIC18F87J50 FS USB Plug-In Module, or
 				Explorer 16 + PIC24 USB PIM.  The firmware may be
 				modified for use on other USB platforms by editing the
 				HardwareProfile.h file.
 Complier:  	Microchip C18 (for PIC18) or C30 (for PIC24)
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
********************************************************************/

/** INCLUDES *******************************************************/

#include "Compiler.h"
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_device.h"

#include "USB/usb_function_generic.h"

#include "user.h"
#include "temperature.h"
#include "xlcd.h"
/** V A R I A B L E S ********************************************************/
#pragma udata
BYTE old_sw2,old_sw3;
BYTE counter;
BYTE trf_state;
BYTE temp_mode;

#if defined(__18F14K50) || defined(__18F13K50) || defined(__18LF14K50) || defined(__18LF13K50) 
    #pragma udata usbram2
#elif defined(__18F2455) || defined(__18F2550) || defined(__18F4455) || defined(__18F4550)\
    || defined(__18F2458) || defined(__18F2453) || defined(__18F4558) || defined(__18F4553)
    #pragma udata USB_VARIABLES=0x500
#elif defined(__18F4450) || defined(__18F2450)
    #pragma udata USB_VARIABLES=0x480
#else
    #pragma udata
#endif

DATA_PACKET INPacket;
DATA_PACKET OUTPacket;
#pragma udata

BYTE pTemp;                     // Pointer to current logging position, will
                                // loop to zero once the max index is reached
BYTE valid_temp;                // Keeps count of the valid data points
WORD temp_data[30];             // 30 points of data

USB_HANDLE USBGenericOutHandle = 0;
USB_HANDLE USBGenericInHandle = 0;

BOOL blinkStatusValid = TRUE;

// Timer0 - 1 second interval setup.
// Fosc/4 = 12MHz
// Use /256 prescalar, this brings counter freq down to 46,875 Hz
// Timer0 should = 65536 - 46875 = 18661 or 0x48E5
#define TIMER0L_VAL         0xE5
#define TIMER0H_VAL         0x48

#define CLEAR_DISPLAY 0x01 
#define FOUR_BIT   0b00101111
#define DON         0b00001111  /* Display on      */
#define DOFF        0b00001011  /* Display off     */
#define CURSOR_ON   0b00001111  /* Cursor on       */
#define CURSOR_OFF  0b00001101  /* Cursor off      */
#define BLINK_ON    0b00001111  /* Cursor Blink    */
#define BLINK_OFF   0b00001110  /* Cursor No Blink */
#define LINES_5X7  0b00111011


unsigned int eeadr;
unsigned char eedata, eechar0, eechar1, eechar2, eechar3, eechar4, eechar5, eechar6;
unsigned char b, b0, b1, b2, b3, b4, b5, b6, b7, b8, b9, b10, b11, b12, b13, b14, b15, b16, b17, tx_buf[10], rdy, ack1, ack2, page, charr[4], gainst, cct, cci;
signed long rez1, rez2, rez3, rez4, rez, rezz, calib, rezult2, rezult3, constt3;
signed long rezult, average1, average2, average3, average4;
double constt1, constt2;

/** P R I V A T E  P R O T O T Y P E S ***************************************/



void ServiceRequests(void);
void init(void);
void writeEEPROM (unsigned int eeadr, unsigned char eedata);
unsigned char readEEPROM (unsigned int eeadr);
//void cantar(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{

    
    #if defined(__18CXX)
    /* Init Timer0 for data logging interval (every 1 second) */
    T0CON = 0b10010111;
    /* Timer0 is already enabled by default */
    #elif defined(__C30__)
    #endif

    blinkStatusValid = TRUE;

}//end UserInit


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    //Blink the LEDs according to the USB device status
 

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

    //respond to any USB commands that might have come over the bus
	PollTempOnHPCExplorer();			
    ServiceRequests();
    
    

    
    //if we are logging temp data
   
}//end ProcessIO

/******************************************************************************
 * Function:        void ResetTempLog(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    pTemp = 0; valid_temp = 0;
 *
 * Overview:        This function resets the temperature logging variables
 *
 * Note:            None
 *****************************************************************************/
void ResetTempLog(void)
{
    pTemp = 0;
    valid_temp = 0;
}//end ResetLog

void ServiceRequests(void)
{
    BYTE index;
    unsigned char i;
    //Check to see if data has arrived
    if(!USBHandleBusy(USBGenericOutHandle))
    {        
        //if the handle is no longer busy then the last
        //transmission is complete
       
        counter = 0;

        INPacket.CMD=OUTPacket.CMD;
        INPacket.len=OUTPacket.len;

        //process the command
      //  switch(OUTPacket.CMD)
       switch(RD_POT)
        {
            case READ_VERSION:
                //dataPacket._byte[1] is len
                
                counter=0x04;
                break;

            case ID_BOARD:
                counter = 0x01;
            
                break;

            case UPDATE_LED:
                #if defined(PIC18F87J50_PIM) || defined(PIC18F46J50_PIM)
                    blinkStatusValid = FALSE;
                #endif
                // LED1 & LED2 are used as USB event indicators.
               
                    counter = 0x01;
               
                break;
                
            case SET_TEMP_REAL:
                temp_mode = TEMP_REAL_TIME;
                ResetTempLog();
                counter = 0x01;
                break;

            case RD_TEMP:
               
                
                    counter=3;
                //end if

                break;

            case SET_TEMP_LOGGING:
             
                counter=0x01;
                break;

            case RD_TEMP_LOGGING:
             
                counter=0x01;
               
                break;

            case RD_POT:
                {
                    WORD_VAL w;
					
					eechar0=OUTPacket._byte[2]-0x30;
                    eechar1=OUTPacket._byte[3]-0x30;
                    eechar2=OUTPacket._byte[4]-0x30;
                    eechar3=OUTPacket._byte[5]-0x30;
                    eechar4=OUTPacket._byte[6]-0x30;
                    eechar5=OUTPacket._byte[7]-0x30;
					constt2=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
					if(constt2>0.0001)
					{
						if(constt2<99.9999)
						{
						constt1=constt2;
						constt3=constt1*10000;
	
						INTCONbits.GIE=0;
						//>>>>>>>>>>>>
if(gainst == 0)
{
		writeEEPROM(36, eechar0);
		writeEEPROM(37, eechar1);
		writeEEPROM(38, eechar2);
		writeEEPROM(39, eechar3);
		writeEEPROM(40, eechar4);
		writeEEPROM(41, eechar5);
}	

if(gainst == 1)
{
		writeEEPROM(42, eechar0);
		writeEEPROM(43, eechar1);
		writeEEPROM(44, eechar2);
		writeEEPROM(45, eechar3);
		writeEEPROM(46, eechar4);
		writeEEPROM(47, eechar5);
}

if(gainst == 2)
{
		writeEEPROM(48, eechar0);
		writeEEPROM(49, eechar1);
		writeEEPROM(50, eechar2);
		writeEEPROM(51, eechar3);
		writeEEPROM(52, eechar4);
		writeEEPROM(53, eechar5);
}	

if(gainst == 3)
{
		writeEEPROM(54, eechar0);
		writeEEPROM(55, eechar1);
		writeEEPROM(56, eechar2);
		writeEEPROM(57, eechar3);
		writeEEPROM(58, eechar4);
		writeEEPROM(59, eechar5);
}

if(gainst == 4)
{
		writeEEPROM(60, eechar0);
		writeEEPROM(61, eechar1);
		writeEEPROM(62, eechar2);
		writeEEPROM(63, eechar3);
		writeEEPROM(64, eechar4);
		writeEEPROM(65, eechar5);

}	

if(gainst == 5)
{
		writeEEPROM(66, eechar0);
		writeEEPROM(67, eechar1);
		writeEEPROM(68, eechar2);
		writeEEPROM(69, eechar3);
		writeEEPROM(70, eechar4);
		writeEEPROM(71, eechar5);
}
		//>>>>>>>>>>>
		INTCONbits.GIE=1;
}	
}
					//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
					
	


					//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
						
					INPacket._byte[0]='0';
            		INPacket._byte[1]='0';
            		hexdec(rezult);
            		INPacket._byte[2]=tx_buf[0];
            		INPacket._byte[3]=tx_buf[1];
            		INPacket._byte[4]=tx_buf[2];
            		INPacket._byte[5]=tx_buf[3];
            		INPacket._byte[6]=tx_buf[4];
            		INPacket._byte[7]=tx_buf[5];
            		hexdec(rezult3);
            		INPacket._byte[8]=tx_buf[0];
            		INPacket._byte[9]=tx_buf[1];
            		INPacket._byte[10]=tx_buf[2];
            		INPacket._byte[11]=tx_buf[3];
            		INPacket._byte[12]=tx_buf[4];
            		INPacket._byte[13]=tx_buf[5];
            		hexdec(rezz);
            		INPacket._byte[14]=tx_buf[0];
            		INPacket._byte[15]=tx_buf[1];
            		INPacket._byte[16]=tx_buf[2];
            		INPacket._byte[17]=tx_buf[3];
            		INPacket._byte[18]=tx_buf[4];
            		INPacket._byte[19]=tx_buf[5];
            		hexdec(calib);
            		INPacket._byte[20]=tx_buf[0];
            		INPacket._byte[21]=tx_buf[1];
            		INPacket._byte[22]=tx_buf[2];
            		INPacket._byte[23]=tx_buf[3];
            		INPacket._byte[24]=tx_buf[4];
            		INPacket._byte[25]=tx_buf[5];
            		//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            		
if(gainst == 0)
{
eechar0=readEEPROM(36);
eechar1=readEEPROM(37);
eechar2=readEEPROM(38);
eechar3=readEEPROM(39);
eechar4=readEEPROM(40);
eechar5=readEEPROM(41);
}	

if(gainst == 1)
{
eechar0=readEEPROM(42);
eechar1=readEEPROM(43);
eechar2=readEEPROM(44);
eechar3=readEEPROM(45);
eechar4=readEEPROM(46);
eechar5=readEEPROM(47);
}

if(gainst == 2)
{
eechar0=readEEPROM(48);
eechar1=readEEPROM(49);
eechar2=readEEPROM(50);
eechar3=readEEPROM(51);
eechar4=readEEPROM(52);
eechar5=readEEPROM(53);
}	

if(gainst == 3)
{
eechar0=readEEPROM(54);
eechar1=readEEPROM(55);
eechar2=readEEPROM(56);
eechar3=readEEPROM(57);
eechar4=readEEPROM(58);
eechar5=readEEPROM(59);
}

if(gainst == 4)
{
eechar0=readEEPROM(60);
eechar1=readEEPROM(61);
eechar2=readEEPROM(62);
eechar3=readEEPROM(63);
eechar4=readEEPROM(64);
eechar5=readEEPROM(65);

}	

if(gainst == 5)
{
eechar0=readEEPROM(66);
eechar1=readEEPROM(67);
eechar2=readEEPROM(68);
eechar3=readEEPROM(69);
eechar4=readEEPROM(70);
eechar5=readEEPROM(71);
}
					//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
					constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
					constt3=constt1*10000;
					//INTCONbits.GIE=1;
            		hexdec(constt3);
            		INPacket._byte[26]=tx_buf[0];
            		INPacket._byte[27]=tx_buf[1];
            		INPacket._byte[28]=tx_buf[2];
            		INPacket._byte[29]=tx_buf[3];
            		INPacket._byte[30]=tx_buf[4];
            		INPacket._byte[31]=tx_buf[5];
              /* 
                   for(i=0;i<=63;i++)
                    {
	                    INPacket._byte[i] = OUTPacket._byte[i];
	                } 
	                */
                    counter=64;
                }
                break;
                
            case RESET:
                Reset();
                break;
                
            default:
                Nop();
                break;
        }//end switch()
        if(counter != 0)
        {
            if(!USBHandleBusy(USBGenericInHandle))
            {
                USBGenericInHandle = USBGenWrite(USBGEN_EP_NUM,(BYTE*)&INPacket,counter);
            }
        }//end if
        
        //Re-arm the OUT endpoint for the next packet
        USBGenericOutHandle = USBGenRead(USBGEN_EP_NUM,(BYTE*)&OUTPacket,USBGEN_EP_SIZE);
    }//end if

}//end ServiceRequests


void TXbyte(BYTE data)
{
    #if defined(__18CXX)
    while(TXSTAbits.TRMT==0);
    TXREG = data;
    #endif
}//end TXbyte

/** EOF user.c ***************************************************************/
void clear_lcd(int line) 
{ 
char ln; 
int x; 
if (line==2) ln = 0xC0; 
else ln = 0x00; 
while( BusyXLCD() ); 
SetDDRamAddr(ln);   // set line 
for (x=0; x<16; x++) 
{ 
// while( BusyXLCD() ); 
/**********taken out***********/ 
WriteDataXLCD(' ');  // write a blank 
} 
while( BusyXLCD() ); 
SetDDRamAddr(ln);   // set back to beginning 
return; 
} 
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void start(void)
{
SDA=0;
Delay10TCYx(3);
SCL=0;
Delay10TCYx(3);	
}
void stop(void)
{
SDA=0;
Delay10TCYx(3);
SCL=1;
Delay10TCYx(3);
SDA=1;
Delay10TCYx(3);
}		

void one(void)
{
SDA=1;
Delay10TCYx(3);
SCL=1;
Delay10TCYx(6);
SCL=0;
Delay10TCYx(3);
}
void zero(void)
{
SDA=0;
Delay10TCYx(3);
SCL=1;
Delay10TCYx(6);
SCL=0;
Delay10TCYx(3);
}
void read(void)
{
Delay10TCYx(3);
SCL=1;
Delay10TCYx(3);
b=RSDA;
Delay10TCYx(3);
SCL=0;
Delay10TCYx(3);
}			


void write(void)
{
start();
one();      //1
one();      //2
zero();		//3
one();		//4
zero();		//5 adres bit
zero();		//6 adres bit
zero();		//7 adres bit
zero();     //8
DSDA=1;     // 
read();     //9	receive ACK from MCP3421
ack1=b;
DSDA=0;     // 
zero();     //1 RDY
zero();     //2	C1
zero();   	//3 C0
one();     	//4	O/C
one();		//5 S1
one();     //6 S0
one();     	//7 G1
one();      //8 G0
DSDA=1;     // receive
read();     //9	receive ACK from MCP3421
ack2=b;
DSDA=0;     // output
stop();	
}
void write2(void)
{
start();
one();      //1
one();      //2
zero();		//3
one();		//4
zero();		//5 adres bit
zero();		//6 adres bit
zero();		//7 adres bit
zero();     //8
DSDA=1;     // 
read();     //9	receive ACK from MCP3421
ack1=b;
DSDA=0;     // 
zero();     //1 RDY
zero();     //2	C1
zero();   	//3 C0
one();     	//4	O/C
one();		//5 S1
one();     //6 S0
zero();     	//7 G1
zero();      //8 G0
DSDA=1;     // receive
read();     //9	receive ACK from MCP3421
ack2=b;
DSDA=0;     // output
stop();	
}	
void write3(void)
{
start();
one();      //1
one();      //2
zero();		//3
one();		//4
zero();		//5 adres bit
zero();		//6 adres bit
zero();		//7 adres bit
zero();     //8
DSDA=1;     // 
read();     //9	receive ACK from MCP3421
ack1=b;
DSDA=0;     // 
zero();     //1 RDY
zero();     //2	C1
zero();   	//3 C0
one();     	//4	O/C
one();		//5 S1
one();     //6 S0
zero();     	//7 G1
one();      //8 G0
DSDA=1;     // receive
read();     //9	receive ACK from MCP3421
ack2=b;
DSDA=0;     // output
stop();	
}	
void ADCC0(void)
{
start();
one();      //1
one();      //2
zero();		//3
one();		//4
zero();		//5 adres bit
zero();		//6 adres bit
zero();		//7 adres bit
DSDA=1;     // receive
read();     //8
read();     //9
read();     //1
read();     //2
read();     //3
read();     //4
read();		//5
read();     //6
read();     //7
b17=b;      // save bit 17
read();     //8
b16=b;      // save bit 16
DSDA=0;     // transmit ACK
zero();     //9
DSDA=1;      //receive data
read();     //1
b15=b;
read();     //2
b14=b;
read();     //3
b13=b;
read();     //4
b12=b;
read();		//5
b11=b;
read();     //6
b10=b;
read();     //7
b9=b;
read();     //8
b8=b;
DSDA=0;     // transmit ACK
zero();     //9
DSDA=1;      //receive data
read();     //1
b7=b;
read();     //2
b6=b;
read();     //3
b5=b;
read();     //4
b4=b;
read();		//5
b3=b;
read();     //6
b2=b;
read();     //7
b1=b;
read();     //8
b0=b;
DSDA=0;     // transmit ACK
zero();     //9
DSDA=1;      //receive data
read();     //1
rdy=b;
read();     //2
read();     //3
read();     //4
read();		//5
read();     //6
read();     //7
read();     //8
DSDA=0;     // transmit NACK
one();      //9
stop();
//b0=0;
}	
void compute(void)
{
b1=b1*2;
b2=b2*4;
b3=b3*8;
b4=b4*16;
b5=b5*32;
b6=b6*64;
b7=b7*128;
rez1=b0+b1+b2+b3+b4+b5+b6+b7;
b9=b9*2;
b10=b10*4;
b11=b11*8;
b12=b12*16;
b13=b13*32;
b14=b14*64;
b15=b15*128;
rez2=b8+b9+b10+b11+b12+b13+b14+b15;
b17=b17*2;
rez3=b16+b17;
rez=rez3*65536+rez2*256+rez1;
rez4=rez & 0b00000000000000100000000000000000;
if(rez4 == 0)
{
	rezz=rez;
}
else
{

	rezz=rez|0b11111111111111000000000000000000;	
}	
	

	if(SW4==0)
		{
		INTCONbits.GIE=0;
		calib=rezz;
		hexdec(calib);
		//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
		eechar0 = tx_buf[0]-0x30;
		eechar1 = tx_buf[1]-0x30;
		eechar2 = tx_buf[2]-0x30;
		eechar3 = tx_buf[3]-0x30;
		eechar4 = tx_buf[4]-0x30;
		eechar5 = tx_buf[5]-0x30;
		//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
		if(gainst == 0)
{


		writeEEPROM(0, eechar0);
		writeEEPROM(1, eechar1);
		writeEEPROM(2, eechar2);
		writeEEPROM(3, eechar3);
		writeEEPROM(4, eechar4);
		writeEEPROM(5, eechar5);

}	

if(gainst == 1)
{


		writeEEPROM(6, eechar0);
		writeEEPROM(7, eechar1);
		writeEEPROM(8, eechar2);
		writeEEPROM(9, eechar3);
		writeEEPROM(10, eechar4);
		writeEEPROM(11, eechar5);

}

if(gainst == 2)
{


		writeEEPROM(12, eechar0);
		writeEEPROM(13, eechar1);
		writeEEPROM(14, eechar2);
		writeEEPROM(15, eechar3);
		writeEEPROM(16, eechar4);
		writeEEPROM(17, eechar5);

}	

if(gainst == 3)
{



		writeEEPROM(18, eechar0);
		writeEEPROM(19, eechar1);
		writeEEPROM(20, eechar2);
		writeEEPROM(21, eechar3);
		writeEEPROM(22, eechar4);
		writeEEPROM(23, eechar5);

}

if(gainst == 4)
{



		writeEEPROM(24, eechar0);
		writeEEPROM(25, eechar1);
		writeEEPROM(26, eechar2);
		writeEEPROM(27, eechar3);
		writeEEPROM(28, eechar4);
		writeEEPROM(29, eechar5);

}	

if(gainst == 5)
{

//	constt1=1.5853;

		writeEEPROM(30, eechar0);
		writeEEPROM(31, eechar1);
		writeEEPROM(32, eechar2);
		writeEEPROM(33, eechar3);
		writeEEPROM(34, eechar4);
		writeEEPROM(35, eechar5);

}
		//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
		INTCONbits.GIE=1;
		LEDD11=1;
		}
	else
		{	
		LEDD11=0;
		}
	rezult=rezz-calib;	

//	constt2=constt1;	
	rezult2=rezult*constt1;		
}	
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void init(void)
{
ADCON1=0b00001111;
CMCON=0b00000111;
CVRCON=0;
UCON=0;
CCP1CON=0;
CCP2CON=0;
DLEDG=0;
DLEDR=0;
DLEDB=0;
LEDD11=0;
LEDD12=0;
LEDD13=0;
DSDA=0;
DSCL=0;
SDA=1;
SCL=1;
DSW2=1;
DSW3=1;
DSW4=1;
calib=0;
page=1;
Delay10TCYx(100);
write();
constt1=1.1805;

charr[0]='1';				//when first powered the weight scale will be in "1PGA" mode: no averaging and internal PGA at maximum gain
charr[1]='P';
charr[2]='g';
charr[3]='a';
gainst=0;
cct=0;
cci=0;
INTCONbits.GIE=0;	
eechar0=readEEPROM(0);		//loading the calibration constants from eeprom for the "1PGA" mode
eechar1=readEEPROM(1);
eechar2=readEEPROM(2);
eechar3=readEEPROM(3);
eechar4=readEEPROM(4);
eechar5=readEEPROM(5);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(36);
eechar1=readEEPROM(37);
eechar2=readEEPROM(38);
eechar3=readEEPROM(39);
eechar4=readEEPROM(40);
eechar5=readEEPROM(41);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;

INTCONbits.GIE=1;
}	

/********************************************************************************/
void cantar(void)
{
	

LEDD12=0;
ADCC0();
if(charr[0] == '1')		//charr[0] is the number that in indicating the averaging on the LCD is eider "1" - no averaging, or "4"- averagin on four samples
{
	if(rdy==1)
	{
	LEDD13=0;	
	}
	else
	{
	LEDD13=1;
	compute();
	rezult3=rezult2;				//the final value is not averaged
	}
}
else
{
	if(rdy==1)
	{
	LEDD13=0;	
	}
	else
	{
	LEDD13=1;
	compute();
	if(cct==0)
	{
		average1=rezult2;
		cct=1;
	}	
	else
	{
		if(cct==1)
		{
			average2=rezult2;
			cct=2;
		}	
		else	
		{
		if(cct==2)
		{
			average3=rezult2;
			cct=3;
		}	
		else
		{
			cct=0;
			average4=rezult2;
			rezult2=average1+average2+average3+average4;
			rezult3=rezult2/4;									//the final value is averaged 4 times
		}	
	}	
	}	
	}
}	
if( page==1 )
{		
while(BusyXLCD()); 
clear_lcd(1); 

putcXLCD('W');  						//LCD will indicate the measured weight in gramms
putcXLCD('e'); 
putcXLCD('i'); 
putcXLCD('g'); 
putcXLCD('h');  
putcXLCD('t'); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(charr[0]); 
putcXLCD(charr[1]); 
putcXLCD(charr[2]); 
putcXLCD(charr[3]); 

//LEDD12=1;
while(BusyXLCD()); 
clear_lcd(2); 
if(rezult3<0)
{
	rezult3=0-rezult3;
	putcXLCD('-'); 
	putcXLCD(' '); 
}	
	else
{
	putcXLCD(' '); 
	putcXLCD(' '); 
}	
hexdec(rezult3);
LEDD12=1;

//putcXLCD(' '); 
//putcXLCD(' '); 
putcXLCD(tx_buf[0]);  
putcXLCD(tx_buf[1]); 
putcXLCD(tx_buf[2]);  
putcXLCD(tx_buf[3]); 
putcXLCD('.');
putcXLCD(tx_buf[4]); 
putcXLCD(tx_buf[5]);
putcXLCD('g' );  
}

	if(page==2)
	{

while(BusyXLCD()); 
clear_lcd(1); 

putcXLCD('A');  			//LCD will indicate the code from the puput of the ADC minus the offset constant
putcXLCD('D'); 
putcXLCD('C'); 
putcXLCD(' '); 
putcXLCD('c');  
putcXLCD('o'); 
putcXLCD('d'); 
putcXLCD('e'); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(charr[0]); 
putcXLCD(charr[1]); 
putcXLCD(charr[2]); 
putcXLCD(charr[3]); 


while(BusyXLCD()); 
clear_lcd(2); 
if(rezult<0)
{
	rezult=0-rezult;
	putcXLCD('-'); 
	putcXLCD(' '); 
}	
	else
{
	putcXLCD(' '); 
	putcXLCD(' '); 
}	
hexdec(rezult);
//putcXLCD(' '); 
//putcXLCD(' '); 
putcXLCD(tx_buf[0]);  
putcXLCD(tx_buf[1]); 
putcXLCD(tx_buf[2]);  
putcXLCD(tx_buf[3]); 
putcXLCD(tx_buf[4]); 
putcXLCD(tx_buf[5]); 		
}

if(page==3)
{

while(BusyXLCD()); 
clear_lcd(1); 

putcXLCD('A');  		//LCD will indicate the sample as it is aquired by the ADC
putcXLCD('D'); 
putcXLCD('C'); 
putcXLCD(' '); 
putcXLCD('o');  
putcXLCD('u'); 
putcXLCD('t'); 
putcXLCD('p'); 
putcXLCD('u');
putcXLCD('t');  
putcXLCD(' '); 
putcXLCD(' '); 
putcXLCD(charr[0]); 
putcXLCD(charr[1]); 
putcXLCD(charr[2]); 
putcXLCD(charr[3]); 

while(BusyXLCD()); 
clear_lcd(2); 
if(rezz<0)
{
	rezz=0-rezz;
	putcXLCD('-'); 
	putcXLCD(' '); 
}	
	else
{
	putcXLCD(' '); 
	putcXLCD(' '); 
}	
hexdec(rezz);
//putcXLCD(' '); 
//putcXLCD(' '); 
putcXLCD(tx_buf[0]);  
putcXLCD(tx_buf[1]); 
putcXLCD(tx_buf[2]);  
putcXLCD(tx_buf[3]); 
putcXLCD(tx_buf[4]); 
putcXLCD(tx_buf[5]); 		
}

if(page==4)
{

while(BusyXLCD()); 
clear_lcd(1); 

putcXLCD('C');  		//LCD will indicate the offset calibration constant
putcXLCD('a'); 
putcXLCD('l'); 
putcXLCD('i');  
putcXLCD('b'); 
putcXLCD('r'); 
putcXLCD('a'); 
putcXLCD('t');
putcXLCD('i'); 
putcXLCD('o');  
putcXLCD('n'); 
putcXLCD(' '); 
putcXLCD(charr[0]); 
putcXLCD(charr[1]); 
putcXLCD(charr[2]); 
putcXLCD(charr[3]); 

while(BusyXLCD()); 
clear_lcd(2); 
if(calib<0)
{
	calib=0-calib;
	putcXLCD('-'); 
	putcXLCD(' '); 
}	
	else
{
	putcXLCD(' '); 
	putcXLCD(' '); 
}	
hexdec(calib);
//putcXLCD(' '); 
//putcXLCD(' '); 
putcXLCD(tx_buf[0]);  
putcXLCD(tx_buf[1]); 
putcXLCD(tx_buf[2]);  
putcXLCD(tx_buf[3]); 
putcXLCD(tx_buf[4]); 
putcXLCD(tx_buf[5]); 		
}
if(SW2==0 && page > 0 )
{
	page--;

}	
if(SW2==0 && page == 0)
{
	page=4;

}	
if(SW3==0 && gainst == 0)
{
	write2();

INTCONbits.GIE=0;	

eechar0=readEEPROM(6);			//reading the calibration constants from EEPROM for "1GLo" mode - no averaging, extrnal gain 50x
eechar1=readEEPROM(7);
eechar2=readEEPROM(8);
eechar3=readEEPROM(9);
eechar4=readEEPROM(10);
eechar5=readEEPROM(11);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(42);
eechar1=readEEPROM(43);
eechar2=readEEPROM(44);
eechar3=readEEPROM(45);
eechar4=readEEPROM(46);
eechar5=readEEPROM(47);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;

INTCONbits.GIE=1;	
gainst=1;
charr[0]='1'; 
charr[1]='G'; 
charr[2]='L'; 
charr[3]='0'; 
}	
else
{
if(SW3==0 && gainst == 1)
{
	write3();
//	constt1=4.99;
INTCONbits.GIE=0;	

eechar0=readEEPROM(12);				//reading calibration constants from EEPROM for "1GHi" mode - no averaging external gain 250x
eechar1=readEEPROM(13);
eechar2=readEEPROM(14);
eechar3=readEEPROM(15);
eechar4=readEEPROM(16);
eechar5=readEEPROM(17);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(48);
eechar1=readEEPROM(49);
eechar2=readEEPROM(50);
eechar3=readEEPROM(51);
eechar4=readEEPROM(52);
eechar5=readEEPROM(53);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;

INTCONbits.GIE=1;
gainst=2;
charr[0]='1'; 
charr[1]='G'; 
charr[2]='H'; 
charr[3]='i'; 
}
else
{
if(SW3==0 && gainst == 2)
{
write();
//	constt1=2.2286;
INTCONbits.GIE=0;

eechar0=readEEPROM(18);			//reading calibration constants for "4PGA" mode - data averaged on 4 samples and ADC PGA is having maximum gain
eechar1=readEEPROM(19);
eechar2=readEEPROM(20);
eechar3=readEEPROM(21);
eechar4=readEEPROM(22);
eechar5=readEEPROM(23);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(54);
eechar1=readEEPROM(55);
eechar2=readEEPROM(56);
eechar3=readEEPROM(57);
eechar4=readEEPROM(58);
eechar5=readEEPROM(59);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;

INTCONbits.GIE=1;
gainst=3;
charr[0]='4'; 
charr[1]='P'; 
charr[2]='g'; 
charr[3]='a'; 
}	
else
{
if(SW3==0 && gainst == 3)
{
	write2();
//	constt1=11.805;
INTCONbits.GIE=0;	

eechar0=readEEPROM(24);				//reading calibration constants from EEPROM for "4GLo" mode - data averaged on 4 samples and external gain 50x
eechar1=readEEPROM(25);
eechar2=readEEPROM(26);
eechar3=readEEPROM(27);
eechar4=readEEPROM(28);
eechar5=readEEPROM(29);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(60);
eechar1=readEEPROM(61);
eechar2=readEEPROM(62);
eechar3=readEEPROM(63);
eechar4=readEEPROM(64);
eechar5=readEEPROM(65);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;

INTCONbits.GIE=1;
gainst=4;
charr[0]='4'; 
charr[1]='G'; 
charr[2]='L'; 
charr[3]='o'; 
}
else
{
if(SW3==0 && gainst == 4)
{
	write3();
//	constt1=4.99;
INTCONbits.GIE=0;	

eechar0=readEEPROM(30);					//reading calibration constant from EEPROM for "4GHi" mode - data averaged on 4 samples and external gain 250x
eechar1=readEEPROM(31);
eechar2=readEEPROM(32);
eechar3=readEEPROM(33);
eechar4=readEEPROM(34);
eechar5=readEEPROM(35);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(66);
eechar1=readEEPROM(67);
eechar2=readEEPROM(68);
eechar3=readEEPROM(69);
eechar4=readEEPROM(70);
eechar5=readEEPROM(71);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;

INTCONbits.GIE=1;
gainst=5;
charr[0]='4'; 
charr[1]='G'; 
charr[2]='H'; 
charr[3]='i'; 
}	
else
{
if(SW3==0 && gainst == 5)
{
	write();
//	constt1=1.5853;
INTCONbits.GIE=0;


	
eechar0=readEEPROM(0);						//reading calibration constants from EEPROM for "1PGA" mode - no averaging and ADC PGA at maximum gain 8x
eechar1=readEEPROM(1);
eechar2=readEEPROM(2);
eechar3=readEEPROM(3);
eechar4=readEEPROM(4);
eechar5=readEEPROM(5);
calib=100000*eechar0+10000*eechar1+1000*eechar2+100*eechar3+10*eechar4+eechar5;

eechar0=readEEPROM(36);
eechar1=readEEPROM(37);
eechar2=readEEPROM(38);
eechar3=readEEPROM(39);
eechar4=readEEPROM(40);
eechar5=readEEPROM(41);
constt1=10*eechar0+eechar1+0.1*eechar2+0.01*eechar3+0.001*eechar4+0.0001*eechar5;
constt3=constt1*10000;


INTCONbits.GIE=1;
gainst=0;
charr[0]='1'; 
charr[1]='P'; 
charr[2]='g'; 
charr[3]='a'; 
}
}
}
}
}
}

Delay10KTCYx(255);

} 
/********************************************************************************/
/******************************************************************/ 
/* 63 */ void writeEEPROM (unsigned int eeadr, unsigned char eedata)
/* 65 */ {
/* 75 */ 	EEADR = eeadr; 				/* WRITE step #3 */
/* 76 */ 	EEDATA = eedata; 	/* WRITE step #4 */
/* 73 */ 	EECON1bits.EEPGD = 0; 	/* WRITE step #1 */
			EECON1bits.CFGS = 0;
/* 74 */ 	EECON1bits.WREN = 1; 	/* WRITE step #2 */
	//		INTCONbits.GIE=0;       //stop interupts >>>>>>>>>>>>>>
/* 77 */ 	EECON2 = 0x55; 			/* WRITE step #5 */
/* 78 */ 	EECON2 = 0xaa; 			/* WRITE step #6 */
/* 79 */ 	EECON1bits.WR = 1; 		/* WRITE step #7 */
/* 80 */ 	while (!PIR2bits.EEIF) {}	/* WRITE step #8 */
/* 82 */ 	PIR2bits.EEIF = 0; 		/* WRITE step #9 */
	//		INTCONbits.GIE=1;       //start interupts >>>>>>>>>>>>>>
/* 86 */ }

unsigned char readEEPROM (unsigned int eeadr)
{
unsigned char eedata;
/* 95 */ 	EECON1bits.EEPGD = 0; 	/* READ step #1 */
/* 96 */ 	EEADR = eeadr; 			/* READ step #2 */
/* 97 */ 	EECON1bits.RD = 1; 		/* READ step #3 */
/* 98 */ 	eedata = EEDATA; 		/* READ step #4 */
			return (eedata);
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

